﻿/*
Brett Andrews: 100586132
Ahmad Askri: 100587147

This script instantiates the controller png and its buttons and displays them to the screen. 
It checks for the number of controllers connected and updates the display accordingly.
*/

using System.Collections;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System;
using UnityEngine;
using UnityEngine.UI;

public class Controller : MonoBehaviour
{
    [DllImport("ControllerTest")]
    public static extern void init();

    [DllImport("ControllerTest")]
    public static extern int numGamePads();

    [DllImport("ControllerTest")]
    public static extern bool isConnected(int gamepadIndex);

    [DllImport("ControllerTest")]
    public static extern bool getButtonDown(int gamepadIndex, int Button);

    [DllImport("ControllerTest")]
    public static extern bool getButtonUp(int gamepadIndex, int Button);

    [DllImport("ControllerTest")]
    public static extern bool getButtonPressed(int gamepadIndex, int Button);

    [DllImport("ControllerTest")]
    public static extern bool getButtonReleased(int gamepadIndex, int Button);

    [DllImport("ControllerTest")]
    public static extern void gamePadUpdate();

    [DllImport("ControllerTest")]
    public static extern float getAxis(int gamepadIndex, int button, int axis = 0);

    public GameObject[] buttons = new GameObject[18];

    public int currentNumber = -1;

    public GameObject controllerPic;

    int numberOfConnectedGamepads = 0;

    float moveX = 549.5f;
    float moveY = 420;

    public enum button
    {
        A, B, X, Y, RB, LB, ST, BACK, RS, LS, DPR, DPL, DPU, DPD, LEFT_STICK, RIGHT_STICK, LT, RT
    }

    Vector3[] RSBase = new Vector3[16];
    Vector3[] LSBase = new Vector3[16];

    // Use this for initialization
    void Start()
    {
        init();

        controllerPic = GameObject.Find("ControllerPicture");

        for (int i = 0; i < buttons.Length; i++) //This forloop searches for the gameobjects in the scene that hold the button sprites
        {
            buttons[i] = GameObject.Find(i.ToString());
            buttons[i].name = buttons[i].name + "0"; //Player number is added to the gameobjects to distinguish them for each player
            buttons[i].GetComponent<SpriteRenderer>().enabled = false; 
        }
        for (int i = 1; i < 4; i++)
        {
            GameObject ctrlPicClone = Instantiate(controllerPic, GameObject.Find("Canvas").transform); //Instantiate the controller png 4 times 
            if (i < 3) //the controller png's x and y position is changed each time to avoid overlapping
            {
                ctrlPicClone.transform.position = new Vector3(controllerPic.transform.position.x + (moveX * i), controllerPic.transform.position.y, controllerPic.transform.position.z);
            }
            else
            {
                ctrlPicClone.transform.position = new Vector3(controllerPic.transform.position.x + (moveX * (i - 3)), controllerPic.transform.position.y - moveY, controllerPic.transform.position.z);
            }
            ctrlPicClone.name = "ControllerPicture" + i.ToString();
            //ctrlPicClone.transform.position = new Vector3(-28.47f, 43, 87.876f);
            for (int k = 0; k < buttons.Length; k++)
            {
                GameObject clone = Instantiate(buttons[k], GameObject.Find("Canvas").transform); //Instantiate all the buttons stored in the button array
                if (i < 3)//the buttons' x and y position is changed each time to avoid overlapping
                {
                    clone.transform.position = new Vector3(buttons[k].transform.position.x + (moveX * i), buttons[k].transform.position.y, buttons[k].transform.position.z);
                }
                else
                {
                    clone.transform.position = new Vector3(buttons[k].transform.position.x + (moveX * (i - 3)), buttons[k].transform.position.y - moveY, buttons[k].transform.position.z);
                }
                clone.name = (k.ToString() + i.ToString());
            }
        }

        for (int i = 0; i < 4; i++) //The resting position of the right analog stick and the left analog stick is set
        {
            RSBase[i] = GameObject.Find(((int)button.RS).ToString() + i.ToString()).transform.position;
            LSBase[i] = GameObject.Find(((int)button.LS).ToString() + i.ToString()).transform.position;
        }
    }

    // Update is called once per frame
    void Update()
    {
        numberOfConnectedGamepads = 0; //number of gamepads connected is checked and updated each frame to see if any connect or disconnet
        for (int i = 0; i < numGamePads(); i++)
        {
            if (isConnected(i))
            {
                print("GamePad " + i + " is Connected");
                numberOfConnectedGamepads++;
            }
        }

        GameObject.Find("NumOfGamepads").GetComponent<Text>().text = "Number of Controllers Connected: " + numberOfConnectedGamepads.ToString(); // Number of connected controllers is displayed on the screen

        for (int i = 0; i < numberOfConnectedGamepads; i++)
        {
            GameObject.Find(((int)button.RS).ToString() + i.ToString()).transform.position = RSBase[i];//Resting position for the analog sticks
            GameObject.Find(((int)button.LS).ToString() + i.ToString()).transform.position = LSBase[i];//Resting position for the analog sticks
            GameObject.Find(((int)button.RS).ToString() + i.ToString()).GetComponent<SpriteRenderer>().color = Color.white; //analog stick colour is set to white when not pressed
            GameObject.Find(((int)button.LS).ToString() + i.ToString()).GetComponent<SpriteRenderer>().color = Color.white; //analog stick colour is set to white when not pressed
            for (int k = 0; k < buttons.Length; k++)
            {
                GameObject.Find(k.ToString() + i.ToString()).GetComponent<SpriteRenderer>().enabled = false; //Unless Pressed, all buttons are not drawn
                if (getButtonDown(i, k) && currentNumber != k)
                {
                    GameObject.Find(k.ToString() + i.ToString()).GetComponent<SpriteRenderer>().enabled = true; //Pressed button is drawn
                    if (k == (int)button.LS) //Analog stick turns red when pressed
                    {
                        GameObject.Find(k.ToString() + i.ToString()).GetComponent<SpriteRenderer>().color = Color.red;
                    }
                    else if (k == (int)button.RS) //Analog stick turns red when pressed
                    {
                        GameObject.Find(k.ToString() + i.ToString()).GetComponent<SpriteRenderer>().color = Color.red;
                    }
                }
            }

            GameObject.Find(((int)button.RS).ToString() + i.ToString()).GetComponent<SpriteRenderer>().enabled = true; //Analog stick are always true so you can see if they're being pressed or not
            GameObject.Find(((int)button.LS).ToString() + i.ToString()).GetComponent<SpriteRenderer>().enabled = true;


            if (getAxis(i, (int)button.RT) > 0.1f) //Display the Right Trigger and the Left Trigger if they are being pressed even a little
            {
                GameObject.Find(((int)button.RT).ToString() + i.ToString()).GetComponent<SpriteRenderer>().enabled = true;
            }
            if (getAxis(i, (int)button.LT) > 0.1f)
            {
                GameObject.Find(((int)button.LT).ToString() + i.ToString()).GetComponent<SpriteRenderer>().enabled = true;
            }

            //When the analog sticks are moved, the position of the sprite is changed. The value is multiplied by a scalar to make it more noticible
            //Because the analog sticks are 'chained' to their base position, they dont move away and just move around their base point
            GameObject.Find(((int)button.LS).ToString() + i.ToString()).transform.position += new Vector3(getAxis(i, (int)button.LEFT_STICK, (int)button.X), getAxis(i, (int)button.LEFT_STICK, (int)button.Y), 0) * 20;
            GameObject.Find(((int)button.RS).ToString() + i.ToString()).transform.position += new Vector3(getAxis(i, (int)button.RIGHT_STICK, (int)button.X), getAxis(i, (int)button.RIGHT_STICK, (int)button.Y), 0) * 20;
        }
        //updates the previous state of the buttons for pressed and released
        gamePadUpdate();
    }

}
